#!/usr/bin/env python
#
# packagesync.py - sync installed packages across several computers.
#
# Copyright 2010 Duane Hinnen
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import tempfile
import sys
import os

from desktopcouch.records.server import CouchDatabase
from desktopcouch.records.record import Record as CouchRecord
import commands

database = CouchDatabase("packages", create=True)
databaserecord = ""
RECORD_TYPE = 'http://www.okiebuntu.homelinux.com/packagesync'
EMACS_CONFIG = os.path.expanduser("~") + "/.emacs"
VIM_CONFIG = os.path.expanduser("~") + "/.vimrc"
ZSH_CONFIG = os.path.expanduser("~") + "/.zshrc"
BASH_CONFIG = os.path.expanduser("~") + "/.bashrc"

def makeDB():
    '''
    gets list of package selections. Cleans up the output.
    saves the data to a CouchDB.
    '''
    print "making database"
    getsel_unfor = commands.getoutput("dpkg --get-selections '*'")
    search(getsel_unfor)
    getsel = getsel_unfor.replace("\t"," ") #.replace("\n"," ")
    
    record_one = CouchRecord({  "_id"       :  "deb"     ,
                                "profile"   :  "user001" ,
                                "packages"  :  getsel    ,
                              },
                              record_type = RECORD_TYPE)

    databaserecord = database.put_record(record_one)


def search(search_string):
    if search_string.find("emacs") != -1:
        que = raw_input("You have emacs installed. Would you like to sync your .emacs file as well. (y/n)")
        if que == "y":
                with open(EMACS_CONFIG, 'r') as emacs:
                    er = emacs.read()
                print er


                record_two = CouchRecord({  "_id"       :  "emacs"   ,  
                                            "profile"   :  "user001" ,
                                            "type"      :  "dotfile" ,
                                            "file"      :  er        ,
                                         },
                                         record_type = RECORD_TYPE)

                databaserecord = database.put_record(record_two)
                emacs.close()
        else:
                pass
    if search_string.find("vim") != -1:
        raw_input("You have vim installed. Would you like to sync your .vimrc file as well. (y/n)")

        if que == "y":
                with open(VIM_CONFIG, 'r') as vim:
                    vr = vim.read()
                print vr


                record_two = CouchRecord({  "_id"       :  "vim"   ,  
                                            "profile"   :  "user001" ,
                                            "type"      :  "dotfile" ,
                                            "file"      :  vr        ,
                                         },
                                         record_type = RECORD_TYPE)

                databaserecord = database.put_record(record_two)
                vim.close()
        else:
                pass

    if search_string.find("zsh") != -1:
        raw_input("You have zsh installed. Would you like to sync your .zshrc file as well. (y/n)")

        if que == "y":
                with open(ZSH_CONFIG, 'r') as zsh:
                    zr = zsh.read()
                print zr


                record_two = CouchRecord({  "_id"       :  "zsh"   ,  
                                            "profile"   :  "user001" ,
                                            "type"      :  "dotfile" ,
                                            "file"      :  zr        ,
                                         },
                                         record_type = RECORD_TYPE)

                databaserecord = database.put_record(record_two)
                zsh.close()
        else:
                pass

    if search_string.find("bash") != -1:
        raw_input("You have bash installed. Would you like to sync your .bashrc file as well. (y/n)")

        if que == "y":
                with open(BASH_CONFIG, 'r') as bash:
                    br = bash.read()
                print br


                record_two = CouchRecord({  "_id"       :  "bash"   ,  
                                            "profile"   :  "user001" ,
                                            "type"      :  "dotfile" ,
                                            "file"      :  br        ,
                                         },
                                         record_type = RECORD_TYPE)

                databaserecord = database.put_record(record_two)
                bash.close()
        else:
                pass


    else:
        pass
    

def UseSettings():
    '''
    Create a View. Execute view. Iterate over it.
    Write the package selections to a tmp file.
    Use file to set selections and then intall the packages
    '''
    map_js = 'function(doc){if (doc._id == "deb"){emit(null,doc.packages);}}'
    database.add_view("installed", map_js, None, "getnames")
    result = database.execute_view("installed", "getnames")

    for row in result:
        dpkglist = row.value
    
    tmp_file = open('/tmp/workfile', 'w')
    tmp_file.write(dpkglist)
    os.system ("sudo dpkg --set-selections < /tmp/workfile")
    os.system ("sudo apt-get -y update")
    os.system ("sudo apt-get dselect-upgrade")
    tmp_file.close()


def UseConfig():
    '''
    Create a View. Execute view. Iterate over it.
    Write the data to a .config in users ~/
    '''
#    config_iter = ""
### emacs ###
    map_js2 = 'function(doc){if (doc._id == "emacs"){emit(null,doc.file);}}'
    database.add_view("emacsconfig", map_js2, None, "getfile")
    result = database.execute_view("emacsconfig", "getfile")

    for row in result:
        config_iter = row.value
        print config_iter
    emacs_file = open(os.path.expanduser("~") + "/.emacs2", 'w')
    emacs_file.write(config_iter)
    emacs_file.close()

### vim ####
    map_js2 = 'function(doc){if (doc._id == "vim"){emit(null,doc.file);}}'
    database.add_view("vimconfig", map_js2, None, "getfile")
    result = database.execute_view("vimconfig", "getfile")

    for row in result:
        config_iter = row.value

    vim_file = open(os.path.expanduser("~") + "/.vimrc2", 'w')
    vim_file.write(config_iter)
    vim_file.close()

### bash ####
    map_js2 = 'function(doc){if (doc._id == "bash"){emit(null,doc.file);}}'
    database.add_view("bashconfig", map_js2, None, "getfile")
    result = database.execute_view("bashconfig", "getfile")

    for row in result:
        config_iter = row.value

    bash_file = open(os.path.expanduser("~") + "/.bashrc2", 'w')
    bash_file.write(config_iter)
    bash_file.close()

### zsh ####
    map_js2 = 'function(doc){if (doc._id == "zsh"){emit(null,doc.file);}}'
    database.add_view("zshconfig", map_js2, None, "getfile")
    result = database.execute_view("zshconfig", "getfile")

    for row in result:
        config_iter = row.value

    zsh_file = open(os.path.expanduser("~") + "/.zshrc2", 'w')
    zsh_file.write(config_iter)
    zsh_file.close()



if __name__ == "__main__":
    sr = raw_input("Do you want to save or restore? (s/r): ")
    if sr == "s":
        makeDB()
    elif sr == "r":
        #UseSettings()
        UseConfig()
    else:
        print "please type 's' or 'r'"
    print "finished"
