#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""DocString

DocString Paragraph

"""

import pygtk
pygtk.require('2.0')
import gtk

import os
import apt
import sys
import shutil
import tarfile
from desktopcouch.records.server import CouchDatabase
from desktopcouch.records.record import Record as CouchRecord
from couchdb.client import ResourceConflict

class PluginBase(object):  
    """Template for plugins"""
    cache = apt.Cache()
    value = False
    database = CouchDatabase("packages", create=True)
    RECORD_TYPE = 'http://www.okiebuntu.homelinux.com/packagesync'
    HOME_DIR = os.path.expanduser("~")

    def isInstalled(self):
        """ It search on the apt list if the package is installed"""
        try:
            PluginBase.cache[(self.prog_name).lower()]
            return True
        except:
            return False

    def saveDotFile(self, data):
        database = PluginBase.database
        """Its add the .file to the database in a file key"""
        try:
            record = database.get_record(self.record_name)
            record["type"] = "dotfile"
            record["file"] = data
        except TypeError:
            record = CouchRecord({  "_id"     : self.record_name,
                                    "profile" : "user001",
                                    "type"    : "dotfile",
                                    "file"    : data,
                                    }, PluginBase.RECORD_TYPE)
        try:        
            databaserecord = database.put_record(record)
        except ResourceConflict:
            print >> sys.sdterr, "Conflict error."
            return False
        return databaserecord 

    def attachDirectory(self, path, temp_file = "odcqxusaza.tar"):
        """Its tar the path directory and attach it to the database"""
        if not os.path.exists(path):
            raise IOError("the directory %s does not exist" % path) 
            return False
        database = PluginBase.database
        HOME_DIR = PluginBase.HOME_DIR
        record = database.get_record(self.record_name)
        if len(record.list_attachments()) > 0:
            record.detach(record.list_attachments()[0])
        tar = tarfile.open(HOME_DIR + "/" + temp_file, "w")
        tar.add(path)
        tar.close()
        record.attach(open(HOME_DIR + "/" + temp_file, "r"), temp_file, "tarfile/tar")
        try:
            databaserecord = database.put_record(record)
        except ResourceConflict:
            print >> sys.sdterror, "can't update database, conflict error."
            return False
        os.remove(HOME_DIR + "/" + temp_file)
        return databaserecord

    def recoverDotFile(self):
        database = PluginBase.database
        try:
            record = database.get_record(self.record_name)
            result =  record["file"]
        except TypeError, KeyError:
            print >> sys.stderr, "error while searching for record in db."
            return False
        return result

    def recoverDirectory(self, path, temp_file = "odcqxusaza.tar"):        
        """ recovering directory"""
        database = PluginBase.database
        HOME_DIR = PluginBase.HOME_DIR
        record = database.get_record(self.record_name)
        if not len(record.list_attachments()) > 0:
            return False
        tar = open(HOME_DIR + "/" + temp_file, "w")
        tar.write(record.attachment_data(record.list_attachments()[0])[0])
        tar.close()
        tar = tarfile.open(HOME_DIR + "/" + temp_file, "r")
        tar.extractall("/")
        os.remove(HOME_DIR + "/" + temp_file)
        return True
        
            
        
class Window:

    def callback_o (self, widget, data=None):
        """ Change the value method of each plugin. 
        """
        #d = ("off", "on")[widget.get_active()]
        #print data.name + " " + str(d)
        self.value = widget.get_active()
        data.value = widget.get_active()
        
        
    def backup_click(self, widget, data=None):
        """ It's call for every sync() method on each plugin that has
        the value = True        
        """
        for op in self.options:
            print op.name
            if op.value:
                op.sync()

    def restore_click(self, widget, data=None):
        for op in self.options:                
            print op.name
            if op.value:
                op.restore()

    def delete_event(self, widget, event, data=None):
        gtk.main_quit()
        return False
        
    def __init__(self, options):
        self.options = options
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.connect("delete_event", self.delete_event)
        self.window.set_border_width(20)
        vbox = gtk.VBox(True, 2)
        self.window.add(vbox)
        # Loading the buttons
        for op in options:
            button = gtk.CheckButton(op.name)
            button.connect("toggled", self.callback_o, op)
            vbox.pack_start(button, True, True, 0)
            if not op.isInstalled():
                button.set_sensitive(False)
            button.show()
        
        ##Run Command Button ##
        button = gtk.Button("Make backup")
        button.connect("clicked", self.backup_click)
        vbox.pack_start(button, True, True, 0)
        button.show()

        ## Restore Command Button ##
        button = gtk.Button("Restore")
        button.connect("clicked", self.restore_click)
        vbox.pack_start(button, True, True, 2)
        button.show()

        vbox.show()
        self.window.show()

    def main(self):
        gtk.main()

def loadPlugins():
    """ Search for every *.py file, (except for: __init__.py and *.pyc)
    From each file, it's takes Plugin Class, instance it and retunr in
    in a list.
    """
    objetos = []
    files = (os.listdir("./plugins"))
    files.sort()
    for arch in files:
        if arch.startswith("_") or arch.endswith("pyc"):
            continue
        mod = __import__("plugins." + arch.split(".")[0])
        mod = getattr(mod, arch.split(".")[0])
        objetos.append(mod.Plugin())
    return objetos
    
def run():
    objetos = loadPlugins()
    window = Window(objetos)
    window.main()

if __name__ == "__main__":
    print __doc__
