#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""DocString

DocString Paragraph

"""

__version__ = "$Revision$"
import apt
import os
import re
import commands
import aptsources.sourceslist
from desktopcouch.records.server import CouchDatabase
from desktopcouch.records.record import Record as CouchRecord
from core.stipple import PluginBase


database = CouchDatabase("packages", create=True)
databaserecord = ""
RECORD_TYPE = 'http://www.okiebuntu.homelinux.com/packagesync'


class Plugin(PluginBase):
    def __init__(self):
        self.name = "Sync Packages"
        self.icon = None

    def sync(self):
    
        ''' Get a list of user installed applications. Not apps installed
        by the OS (default apps) but applications installed 
        by the user.'''
            
        cache = apt.Cache()

        default_packages = set()
        # these are false default as you have some alternatives deps that
        # are taken
        # into account like file-roller depends zip | p7zip-full
        # -> p7zip-full won't be listed
        false_defaults = set(['p7zip-full', 'vim-gnome', 'vim'])

        # default whitelist, completed then by ubuntu-restricted-extras deps
        whitelist = set(['flashplugin-nonfree', 'gnash', 'gstreamer0.10-fluendo-mpegdemux', 'swfdec-gnome',
        'swfdec-mozilla', 'ubuntu-restricted-extras'])

        desktop_pkg_file_pattern = '/usr/share/applications/.*\.desktop'
        bin_file_pattern = '/bin/.*'
        sbin_file_pattern = '/sbin/.*'
        usr_bin_file_pattern = '/usr/bin/.*'
        usr_sbin_file_pattern = '/usr/sbin/.*'

        def get_dep_rec_list(root_package, default_packages, recursive=True):
            '''Get list of dep of package_root'''

            for relations in (root_package.candidate.dependencies,
                              root_package.candidate.recommends):
                for dep in relations:
                    for or_dep in dep.or_dependencies:
                        if or_dep.name not in default_packages:
                            default_packages.add(or_dep.name)
                            try:
                                if recursive:
                                    get_dep_rec_list(cache[or_dep.name], default_packages)
                            except KeyError:
                                pass
        get_dep_rec_list(cache['ubuntu-desktop'], default_packages)
        get_dep_rec_list(cache['ubuntu-minimal'], default_packages)
        get_dep_rec_list(cache['ubuntu-standard'], default_packages)
        default_packages -= false_defaults

        #add to the whitelist ubuntu-restricted-extras direct dep
        get_dep_rec_list(cache['ubuntu-restricted-extras'], whitelist, recursive=False)

        # get list of all apps installed
        candidate_pkg = set()
        for pkg in cache:
            if (pkg.is_installed and not pkg.is_auto_installed and not
                pkg.priority in ('required', 'important')):
                if pkg.name in whitelist:
                    candidate_pkg.add(pkg.name)
                else:
                    for pkg_file in pkg.installed_files:
                        if (re.match(desktop_pkg_file_pattern, pkg_file) or
                            re.match(bin_file_pattern, pkg_file) or
                            re.match(sbin_file_pattern, pkg_file) or
                            re.match(usr_bin_file_pattern, pkg_file) or
                            re.match(usr_sbin_file_pattern, pkg_file)):
                            candidate_pkg.add(pkg.name)
                            break

        # archive those not in default set
        pkg_to_archive = set()
        pkg_to_archive = candidate_pkg - default_packages

        getsel = sorted(pkg_to_archive)
        ##this might be able to be reused. Config and package use same DB write
        record_one = CouchRecord({  "_id"       :  "deb"     ,
                                    "profile"   :  "user001" ,
                                    "packages"  :  getsel    ,
                                    },
                                    record_type = RECORD_TYPE)

        databaserecord = database.put_record(record_one)

    def sources(self): 
        ''' write the software sources.list to a CouchBD '''
        lst = []
        src = aptsources.sourceslist.SourcesList()
        
        for obj in src:
            lst.append(str(obj))
            #l1=lst.pop()
            
            #print l1 ## debug

       
        record_two = CouchRecord({  "_id"       :  "sources" ,
                                    "profile"   :  "user001" ,
                                    "file"      :  lst       ,
                                  },
                                  record_type = RECORD_TYPE)

        databaserecord = database.put_record(record_two)

    def restore(self):
        '''
        Create a View. Execute view. Iterate over it.
        Write the package selections to a tmp file.
        Use file to set selections and then intall the packages
        '''
        map_js = 'function(doc){if (doc._id == "deb"){emit(null,doc.packages);}}'
        database.add_view("installed", map_js, None, "getnames")
        result = database.execute_view("installed", "getnames")

        for row in result:
            dpkglist = row.value
        
        tmp_file = open('/tmp/workfile', 'w')
        tmp_file.write(dpkglist)
        os.system ("sudo dpkg --set-selections < /tmp/workfile")
        os.system ("sudo apt-get -y update")
        os.system ("sudo apt-get dselect-upgrade")
        tmp_file.close()

    def search(self):
        pass
    
    def isInstalled(self):
        return True

